@@ -33,6 +33,7 @@ pub const DEFAULT_WORKER_THREADS: usize = 1;
 
 #[derive(Debug, Clone)]
 pub struct PubSubConfig {
+    pub enable_block_subscription: bool,
     pub enable_vote_subscription: bool,
     pub max_active_subscriptions: usize,
     pub queue_capacity_items: usize,
@@ -44,6 +45,7 @@ pub struct PubSubConfig {
 impl Default for PubSubConfig {
     fn default() -> Self {
         Self {
+            enable_block_subscription: false,
             enable_vote_subscription: false,
             max_active_subscriptions: MAX_ACTIVE_SUBSCRIPTIONS,
             queue_capacity_items: DEFAULT_QUEUE_CAPACITY_ITEMS,
@@ -57,6 +59,7 @@ impl Default for PubSubConfig {
 impl PubSubConfig {
     pub fn default_for_tests() -> Self {
         Self {
+            enable_block_subscription: false,
             enable_vote_subscription: false,
             max_active_subscriptions: MAX_ACTIVE_SUBSCRIPTIONS,
             queue_capacity_items: DEFAULT_TEST_QUEUE_CAPACITY_ITEMS,
@@ -142,6 +145,9 @@ fn count_final(params: &SubscriptionParams) {
         SubscriptionParams::Vote => {
             inc_new_counter_info!("rpc-pubsub-final-votes", 1);
         }
+        SubscriptionParams::Block(_) => {
+            inc_new_counter_info!("rpc-pubsub-final-slot-txs", 1);
+        }
     }
 }
 
@@ -187,16 +193,17 @@ pub struct TestBroadcastReceiver {
 #[cfg(test)]
 impl TestBroadcastReceiver {
     pub fn recv(&mut self) -> String {
-        use {
-            std::{
-                thread::sleep,
-                time::{Duration, Instant},
-            },
-            tokio::sync::broadcast::error::TryRecvError,
+        return match self.recv_timeout(std::time::Duration::from_secs(5)) {
+            Err(err) => panic!("broadcast receiver error: {}", err),
+            Ok(str) => str,
         };
+    }
 
-        let timeout = Duration::from_secs(5);
-        let started = Instant::now();
+    pub fn recv_timeout(&mut self, timeout: std::time::Duration) -> Result<String, String> {
+        use std::thread::sleep;
+        use tokio::sync::broadcast::error::TryRecvError;
+
+        let started = std::time::Instant::now();
 
         loop {
             match self.inner.try_recv() {
@@ -206,17 +213,16 @@ impl TestBroadcastReceiver {
                         started.elapsed().as_millis()
                     );
                     if let Some(json) = self.handler.handle(notification).expect("handler failed") {
-                        return json.to_string();
+                        return Ok(json.to_string());
                     }
                 }
                 Err(TryRecvError::Empty) => {
-                    assert!(
-                        started.elapsed() <= timeout,
-                        "TestBroadcastReceiver: no data, timeout reached"
-                    );
-                    sleep(Duration::from_millis(50));
+                    if started.elapsed() > timeout {
+                        return Err("TestBroadcastReceiver: no data, timeout reached".into());
+                    }
+                    sleep(std::time::Duration::from_millis(50));
                 }
-                Err(err) => panic!("broadcast receiver error: {}", err),
+                Err(e) => return Err(e.to_string()),
             }
         }
     }
@@ -230,6 +236,7 @@ pub fn test_connection(
 
     let rpc_impl = RpcSolPubSubImpl::new(
         PubSubConfig {
+            enable_block_subscription: true,
             enable_vote_subscription: true,
             queue_capacity_items: 100,
             ..PubSubConfig::default()
@@ -383,7 +390,10 @@ mod tests {
         },
         std::{
             net::{IpAddr, Ipv4Addr},
-            sync::{atomic::AtomicBool, RwLock},
+            sync::{
+                atomic::{AtomicBool, AtomicU64},
+                RwLock,
+            },
         },
     };
 
@@ -391,6 +401,7 @@ mod tests {
     fn test_pubsub_new() {
         let pubsub_addr = SocketAddr::new(IpAddr::V4(Ipv4Addr::new(0, 0, 0, 0)), 0);
         let exit = Arc::new(AtomicBool::new(false));
+        let max_complete_transaction_status_slot = Arc::new(AtomicU64::default());
         let GenesisConfigInfo { genesis_config, .. } = create_genesis_config(10_000);
         let bank = Bank::new_for_tests(&genesis_config);
         let bank_forks = Arc::new(RwLock::new(BankForks::new(bank)));
@@ -398,6 +409,7 @@ mod tests {
             OptimisticallyConfirmedBank::locked_from_bank_forks_root(&bank_forks);
         let subscriptions = Arc::new(RpcSubscriptions::new_for_tests(
             &exit,
+            max_complete_transaction_status_slot,
             bank_forks,
             Arc::new(RwLock::new(BlockCommitmentCache::new_for_tests())),
             optimistically_confirmed_bank,
